@@ -153,6 +153,10 @@ class PyBulletClient(PyBulletBase, ClientInterface):
         self._cache_dir.cleanup()
         self.disconnect()
 
+    @property
+    def unordered_disabled_collisions(self):
+        return {frozenset(pair) for pair in self.disabled_collisions}
+
     def step_simulation(self):
         """By default, the physics server will not step the simulation,
         unless you explicitly send a ``step_simulation`` command.  This
@@ -162,7 +166,7 @@ class PyBulletClient(PyBulletBase, ClientInterface):
         """
         pybullet.stepSimulation(physicsClientId=self.client_id)
 
-    def load_robot(self, urdf_file, resource_loaders=None):
+    def load_robot(self, urdf_file, resource_loaders=None, concavity=False):
         """Create a pybullet robot using the input urdf file.
 
         Parameters
@@ -174,20 +178,24 @@ class PyBulletClient(PyBulletBase, ClientInterface):
             List of :class:`compas.robots.AbstractMeshLoader` for loading geometry of the robot.  That the
             geometry of the robot model is loaded is required before adding or removing attached collision meshes
             to or from the scene. Defaults to the empty list.
+        concavity : :obj:`bool`
+            When ``False`` (the default), the mesh will be loaded as its
+            convex hull for collision checking purposes.  When ``True``,
+            a non-static mesh will be decomposed into convex parts using v-HACD.
 
         Notes
         -----
         By default, PyBullet will use the convex hull of any mesh loaded from a URDF for collision detection.
         Amending the link tag as ``<link concave="yes" name="<name of link>">`` will make the mesh concave
         for static meshes (see this `example <https://github.com/bulletphysics/bullet3/blob/master/data/samurai.urdf>`_).
-        For non-static concave meshes, replace the OBJ files within the URDF with those generated by ``pybullet.vhacd``.
+        For non-static concave meshes, use the ``concavity`` flag.
         """
         robot_model = RobotModel.from_urdf_file(urdf_file)
         robot = Robot(robot_model, client=self)
         robot.attributes['pybullet'] = {}
         if resource_loaders:
             robot_model.load_geometry(*resource_loaders)
-            self.cache_robot(robot)
+            self.cache_robot(robot, concavity)
         else:
             robot.attributes['pybullet']['cached_robot'] = robot.model
             robot.attributes['pybullet']['cached_robot_filepath'] = urdf_file
@@ -232,7 +240,7 @@ class PyBulletClient(PyBulletBase, ClientInterface):
         self.set_robot_configuration(robot, current_configuration)
         self.step_simulation()
 
-    def cache_robot(self, robot):
+    def cache_robot(self, robot, concavity=False):
         """Saves an editable copy of the robot's model and its meshes
         for shadowing the state of the robot on the PyBullet server.
 
@@ -240,6 +248,10 @@ class PyBulletClient(PyBulletBase, ClientInterface):
         ----------
         robot : :class:`compas_fab.robots.Robot`
             The robot to be saved for use with PyBullet.
+        concavity : :obj:`bool`
+            When ``False`` (the default), the mesh will be loaded as its
+            convex hull for collision checking purposes.  When ``True``,
+            a non-static mesh will be decomposed into convex parts using v-HACD.
 
         Raises
         ------
@@ -256,9 +268,10 @@ class PyBulletClient(PyBulletBase, ClientInterface):
                 shape = element.geometry.shape
                 if isinstance(shape, MeshDescriptor):
                     mesh = shape.geometry
-                    mesh_file_name = str(mesh.guid) + '.stl'
+                    mesh_file_name = str(mesh.guid) + '.obj'
                     fp = os.path.join(self._cache_dir.name, mesh_file_name)
-                    mesh.to_stl(os.path.join(self._cache_dir.name, mesh_file_name), binary=True)
+                    mesh.to_obj(fp)
+                    fp = self._handle_concavity(fp, self._cache_dir.name, concavity, 1, str(mesh.guid))
                     address_dict[shape.filename] = fp
 
         # create urdf with new mesh locations
@@ -461,10 +474,10 @@ class PyBulletClient(PyBulletBase, ClientInterface):
         """
         cached_robot = self.get_cached_robot(robot)
         body_id = self.get_uid(cached_robot)
-        link_names = cached_robot.get_link_names_with_collision_geometry()
+        link_names = [link.name for link in cached_robot.iter_links() if link.collision]
         # check for collisions between robot links
         for link_1_name, link_2_name in combinations(link_names, 2):
-            if {link_1_name, link_2_name} in self.disabled_collisions:
+            if {link_1_name, link_2_name} in self.unordered_disabled_collisions:
                 continue
             link_1_id = self._get_link_id_by_name(link_1_name, cached_robot)
             link_2_id = self._get_link_id_by_name(link_2_name, cached_robot)
@@ -643,11 +656,13 @@ class PyBulletClient(PyBulletBase, ClientInterface):
         return pyb_body_id
 
     @staticmethod
-    def _handle_concavity(tmp_obj_path, tmp_dir, concavity, mass):
+    def _handle_concavity(tmp_obj_path, tmp_dir, concavity, mass, mesh_name=''):
         if not concavity or mass == const.STATIC_MASS:
             return tmp_obj_path
-        tmp_vhacd_obj_path = os.path.join(tmp_dir, 'vhacd_temp.obj')
-        tmp_log_path = os.path.join(tmp_dir, 'log.txt')
+        if mesh_name:
+            mesh_name += '_'
+        tmp_vhacd_obj_path = os.path.join(tmp_dir, mesh_name + 'vhacd_temp.obj')
+        tmp_log_path = os.path.join(tmp_dir, mesh_name + 'log.txt')
         with redirect_stdout():
             pybullet.vhacd(tmp_obj_path, tmp_vhacd_obj_path, tmp_log_path)
         return tmp_vhacd_obj_path
